#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <limits.h>
#include <signal.h>

static char const prog[]="delayrun";

static void alarm_handler(int sig) {
  (void)sig;
}

int main(int argc, char** argv) {
  unsigned long delay=0;
  char const* x;

  x=env_get(var_delay+1);
  if (x!=null) {
    unsigned int const len=scan_ulong(x, &delay);
    if (len==0 || x[len]!='\0')
      strerr_die4x(111, prog, fatal, var_delay, err_malformed);
    if (delay>LONG_MAX)
      strerr_die5x(100, prog, fatal, var_delay, err_oflow, t_long);
  }

  if (x==null || delay!=0) {
    struct itimerval timer;
    sigset_t oldset, alarmset;
    struct sigaction oldact, alarmact;
    timer.it_interval.tv_sec=0;
    timer.it_interval.tv_usec=0;
    timer.it_value.tv_sec=delay;
    timer.it_value.tv_usec=0;
    if (sigprocmask(SIG_BLOCK, null, &alarmset)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_sigmask, err_);
    if (sigaddset(&alarmset, SIGALRM)!=0)
      strerr_die6sys(100, prog, fatal, err_unable, err_addto, err_sigset,
                     err_);
    if (sigprocmask(SIG_BLOCK, &alarmset, &oldset)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_sigmask, err_);
    if (setitimer(ITIMER_REAL, &timer, null)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_timer, err_);
    alarmact.sa_handler=alarm_handler;
    if (sigemptyset(&alarmact.sa_mask)!=0)
      strerr_die6sys(100, prog, fatal, err_unable, err_empty, err_sigset,
                     err_);
    alarmact.sa_flags=0;
    if (sigaction(SIGALRM, &alarmact, &oldact)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_sigact, err_);
    if (sigdelset(&alarmset, SIGALRM)!=0)
      strerr_die6sys(100, prog, fatal, err_unable, err_delfrom, err_sigset,
                     err_);
    if (sigsuspend(&alarmset)!=0 && errno!=EINTR)
      strerr_die5sys(100, prog, fatal, err_unable, err_suspend, err_);
    if (sigaction(SIGALRM, &oldact, null)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_sigact, err_);
    if (sigprocmask(SIG_SETMASK, &oldset, null)!=0)
      strerr_die5sys(100, prog, fatal, err_unable, err_sigmask, err_);
  }

  if (argc<2) _exit(0);
  ++argv;
  pathexec_run(argv[0], (char const**)argv, (char const**)environ);
  switch (errno) {
    default: argc=100; break;
    #ifdef ENOMEM
      case ENOMEM:
    #endif
    #ifdef ETXTBUSY
      case ETXTBUSY:
    #endif
    #ifdef EIO
      case EIO:
    #endif
    #ifdef ENFILE
      case ENFILE:
    #endif
      argc=111;
  }
  strerr_die6sys(argc, prog, fatal, err_unable, err_exec, argv[0], err_);
}
