#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <limits.h>

static char const prog[]="maxinterval";

static char const var_file[]="$MAXINTERVAL_FILE";
static char const var_max[]="$MAXINTERVAL";

int main(int argc, char** argv) {
  struct stat statbuf;
  char const* x;

  if (argc<2)
    strerr_die4x(111, prog, usage, prog, " program [arg ...]");

  x=env_get(var_file+1);
  if (x==null)
    strerr_die4x(111, prog, fatal, var_file, err_notset);

  if (stat(x, &statbuf)!=0) {
    if (errno!=ENOENT) {
      if (errno==ENOMEM) argc=111;
      else argc=100;
      strerr_die6sys(argc, prog, fatal, err_unable, err_stat, x, err_);
    }
  } else {
    time_t const now=time(null);
    unsigned long delay;
    unsigned long max;

    {
      unsigned int len;

      x=env_get(var_max+1);
      if (x==null)
        strerr_die4x(111, prog, fatal, var_max, err_notset);
      len=scan_ulong(x, &max);
      if (len==0 || x[len]!='\0')
        strerr_die4x(111, prog, fatal, var_max, err_malformed);

      x=env_get(var_delay+1);
      if (x!=null) {
        len=scan_ulong(x, &delay);
        if (len==0 || x[len]!='\0')
          strerr_die4x(111, prog, fatal, var_delay, err_malformed);
      }
    }

    if (statbuf.st_mtime<=now) {
      unsigned long const since=now-statbuf.st_mtime;
      if (since>=max) max=0;
      else max-=since;
    } else max+=(unsigned long)(statbuf.st_mtime-now);

    if (x==null || delay>max) delay=max;

    {
      char delaybuf[(sizeof delay*CHAR_BIT+2)/3+1];
      delaybuf[fmt_ulong(delaybuf, delay)]='\0';
      if (pathexec_env(var_delay+1, delaybuf)==0)
        strerr_die6sys(111, prog, fatal, err_unable, err_setvar, var_delay,
                       err_);
    }
  }

  ++argv;
  pathexec((char const**)argv);
  switch (errno) {
    default: argc=100; break;
    #ifdef ENOMEM
      case ENOMEM:
    #endif
    #ifdef ETXTBUSY
      case ETXTBUSY:
    #endif
    #ifdef EIO
      case EIO:
    #endif
    #ifdef ENFILE
      case ENFILE:
    #endif
      argc=111;
  }
  strerr_die6sys(argc, prog, fatal, err_unable, err_exec, argv[0], err_);
}
